@@ -1,5 +1,5 @@
 /* zxiddi.c  -  Discovery Server
- * Copyright (c) 2010 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
+ * Copyright (c) 2010-2011 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.
  * Copyright (c) 2009 Symlabs (symlabs@symlabs.com), All Rights Reserved.
  * Author: Sampo Kellomaki (sampo@iki.fi)
  * This is confidential unpublished proprietary source code of the author.
@@ -9,6 +9,7 @@
  * $Id: zxiddi.c,v 1.2 2009-11-24 23:53:40 sampo Exp $
  *
  * 15.11.2009, created --Sampo
+ * 10.1.2011,  added TrustPDP and CPN support --Sampo
  *
  * See also zxidepr.c for discovery client code.
  *
@@ -24,6 +25,7 @@
 #include "zxidutil.h"
 #include "zxidconf.h"
 #include "saml2.h"
+#include "tas3.h"
 #include "wsf.h"
 #include "c/zx-const.h"
 #include "c/zx-ns.h"
@@ -69,12 +71,16 @@ struct zx_di_QueryResponse_s* zxid_di_query(zxid_conf* cf,zxid_ses* ses,struct z
   char mdpath[ZXID_MAX_BUF];
   char path[ZXID_MAX_BUF];
   char* epr_buf;
+  char* p;
+  char* start;
+  char* lim;
   DIR* dir;
   struct dirent* de;
   struct zx_elem_s* el;
   struct zx_a_Metadata_s* md = 0;  
   struct zx_str* ss;
-  struct zx_str* tt;
+  struct zx_str* svctyp;
+  struct zx_str* prvid;
   struct zx_str* addr = 0;  
   zxid_epr* epr = 0;
   D_INDENT("di_query: ");
@@ -165,6 +171,8 @@ struct zx_di_QueryResponse_s* zxid_di_query(zxid_conf* cf,zxid_ses* ses,struct z
       }
       addr = ZX_GET_CONTENT(epr->Address);
       md = epr->Metadata;
+      svctyp = ZX_GET_CONTENT(md->ServiceType);
+      prvid = ZX_GET_CONTENT(md->ProviderID);
 
       /* Filter by service type */
       
@@ -176,14 +184,13 @@ struct zx_di_QueryResponse_s* zxid_di_query(zxid_conf* cf,zxid_ses* ses,struct z
 	if (!ss || !ss->len)
 	  continue;
 	match = 0;
-	tt = ZX_GET_CONTENT(md->ServiceType);
-	if (!tt || !tt->len) {
+	if (!svctyp || !svctyp->len) {
 	  INFO("EPR missing ServiceType. Rejected. epr_buf(%.*s) file(%s)", epr_len, epr_buf, de->d_name);
 	  ZX_FREE(cf->ctx, epr_buf);
 	  goto next_file;
 	}
-	if (ss->len != tt->len || memcmp(ss->s, tt->s, ss->len)) {
-	  D("%d: Internal svctype(%.*s) does not match desired(%.*s)", n_discovered, tt->len, tt->s, ss->len, ss->s);
+	if (ss->len != svctyp->len || memcmp(ss->s, svctyp->s, ss->len)) {
+	  D("%d: Internal svctype(%.*s) does not match desired(%.*s)", n_discovered, svctyp->len, svctyp->s, ss->len, ss->s);
 	  continue;
 	}
 	D("%d: ServiceType matches. file(%s)", n_discovered, de->d_name);
@@ -205,13 +212,12 @@ struct zx_di_QueryResponse_s* zxid_di_query(zxid_conf* cf,zxid_ses* ses,struct z
 	if (!ss || !ss->len)
 	  continue;
 	match = 0;
-	tt = ZX_GET_CONTENT(md->ProviderID);
-	if (!tt || !tt->len) {
+	if (!prvid || !prvid->len) {
 	  INFO("EPR missing ProviderID. epr_buf(%.*s) file(%s)", epr_len, epr_buf, de->d_name);
 	  break;
 	}
-	if (ss->len != tt->len || memcmp(ss->s, tt->s, ss->len)) {
-	  D("%d: ProviderID(%.*s) does not match desired(%.*s)", n_discovered, tt->len, tt->s, ss->len, ss->s);
+	if (ss->len != prvid->len || memcmp(ss->s, prvid->s, ss->len)) {
+	  D("%d: ProviderID(%.*s) does not match desired(%.*s)", n_discovered, prvid->len, prvid->s, ss->len, ss->s);
 	  continue;
 	}
 	D("%d: ProviderID matches. file(%s)", n_discovered, de->d_name);
@@ -243,17 +249,85 @@ struct zx_di_QueryResponse_s* zxid_di_query(zxid_conf* cf,zxid_ses* ses,struct z
 	ZX_FREE(cf->ctx, epr_buf);
 	goto next_file;
       }
+      
+      /* Check Options, in particular whether Trust parameters are there. */
 
-      /* *** Check Options */
+      if (rs->Options) {
+	for (el = rs->Options->Option;
+	     el && el->g.tok == zx_di_Option_ELEM;
+	     el = (struct zx_elem_s*)el->g.n) {
+	  ss = ZX_GET_CONTENT(el);
+	  if (!ss || !ss->len) {
+	    D("Option element does not have content %p", ss);
+	    continue;
+	  }
+	  p = zx_memmem(ss->s, ss->len, TAS3_TRUST_CTL1_INPUT, sizeof(TAS3_TRUST_CTL1_INPUT)-1);
+	  if (!p) {
+	    D("Option(%.*s) is not trust related", ss->len, ss->s);	    
+	    continue;
+	  }
+	  start = p;
+	  lim = memchr(p+sizeof(TAS3_TRUST_CTL1_INPUT)-1, '&', ss->len - (p - ss->s));
+	  if (!lim) {
+	    lim = ss->s + ss->len;
+	  } else {
+	    while (p = zx_memmem(lim, ss->len - (lim - ss->s), TAS3_TRUST_CTL1_INPUT, sizeof(TAS3_TRUST_CTL1_INPUT)-1)) {
+	      lim = memchr(p+sizeof(TAS3_TRUST_CTL1_INPUT)-1, '&', ss->len - (p - ss->s));
+	      if (!lim) {
+		lim = ss->s + ss->len;
+		break;
+	      }
+	    }
+	  }
+	  
+	  if (cf->trustpdp_url && *cf->trustpdp_url) {
+	    D("Trust related discovery options(%.*s), TRUSTPDP_URL(%s)", lim-start, start, cf->trustpdp_url);
+	    if (zxid_call_trustpdp(cf, 0, ses, cf->pepmap_rsin, start, lim, epr)) {
+	      D("%d: Trust PERMIT. file(%s)", n_discovered, de->d_name);
+	      continue;
+	    } else {
+	      D("%d: Rejected due to Trust DENY. file(%s)", n_discovered, de->d_name);
+	      ZX_FREE(cf->ctx, epr_buf);
+	      goto next_file;
+	    }
+	  } else {
+	    INFO("Trust related discovery options(%.*s), but no TRUSTPDP_URL configured", lim-start, start);
+	    continue;
+	  }
+	}
+      }
 
       /* *** Check Framework */
 
       /* *** Check Action */
 
+      if (cf->cpn_ena) {
 #if 0
-      /* Call Trust and Privacy Negotiation (TrustBuilder), Andreas. */
-      systemf("./tpn-client.sh %s %s %s", idpnid, "urn:idhrxml:cv:update", host);
+	/* Call Trust and Privacy Negotiation (TrustBuilder), Andreas. */
+	systemf("./tpn-client.sh %s %s %s", idpnid, "urn:idhrxml:cv:update", host);
+#else
+	if (svctyp && svctyp->len && prvid && prvid->len) {
+	  ss = zxid_callf(cf, ses, "urn:tas3:cpn-agent",0,0,0,
+		 "<tas3cpn:CPNRequest xmlns:tas3cpn=\"urn:tas3:cpn-agent\">"
+		   "<di:RequestedService xmlns:di=\"urn:liberty:disco:2006-08\">"
+		     "<di:ServiceType>%.*s</di:ServiceType>"
+		     "<di:ProviderID>%.*s</di:ProviderID>"
+		     "<di:Framework version=\"2.0\"/>"
+		     /*"<di:Action>urn:x-foobar:Create</di:Action>"*/
+		   "</di:RequestedService>"
+		 "</tas3cpn:CPNRequest>",
+			  svctyp->len, svctyp->s,
+			  prvid->len, prvid->s);
+	  if (!ss || !ss->s) {
+	    D("CPN returned nothing or emptiness (no CPN agent discoverable?) %p", ss);
+	  } else {
+	    D("CPN returned(%.*s)", ss->len, ss->s);	  
+	  }
+	} else {
+	  ERR("Service Metadata missing ServiceType(%p) or ProviderID(%p)", md->ServiceType, md->ProviderID);
+	}
 #endif
+      }
       ++n_discovered;
       D("%d: DISCOVERED EPR url(%.*s)", n_discovered, addr->len, addr->s);
       if (!zxid_add_fed_tok2epr(cf, ses, epr, 1, logop)) {
@@ -289,3 +363,4 @@ next_file:
 }
 
 /* EOF  --  zxiddi.c */
+
